@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Document;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One JSON Schema document
 
-our $VERSION = '0.514';
+our $VERSION = '0.515';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -39,7 +39,7 @@ has canonical_uri => (
 
 has specification_version => (
   is => 'rwp',
-  isa => Enum([qw(draft7 draft2019-09)]),
+  isa => Enum([qw(draft7 draft2019-09 draft2020-12)]),
 );
 
 has resource_index => (
@@ -190,7 +190,7 @@ JSON::Schema::Modern::Document - One JSON Schema document
 
 =head1 VERSION
 
-version 0.514
+version 0.515
 
 =head1 SYNOPSIS
 
